;;; hs-show-html-mail.el --- View HTML-parts in a graphical browser.

;; Copyright (C) 2005, 2006 Hynek Schlawack

;; Author: Hynek Schlawack <hynek@ularx.de>
;; Version: $Id: hs-show-html-mail.el 13 2006-01-22 12:36:36Z hynek $
;; Keywords: mail news

;; This file is NOT part of GNU Emacs.

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; Scans the actual article in Gnus for HTML-parts and shows them using
;; browse-url.

;;; Code:

(defun gnus-article-browse-html-parts (list)
"View all \"text/html\" parts from LIST.
Recurse into multiparts."

  (let ((showed))
    ;; Find and show the html-parts.
    (dolist (handle list)
      ;; If HTML, show it
      (when (listp handle)
	(cond ((and (bufferp (car handle))
		    (string-match "text/html" (car (mm-handle-type handle))))
	       (let ((tmp-file (format "%s-%s.html"
				   (make-temp-name "/tmp/gnus-html-")
				   (user-login-name))))
		 (mm-save-part-to-file handle tmp-file)
		 (browse-url tmp-file)
		 (setq showed t)))
	      ;; If multipart, recurse
	      ((and (stringp (car handle))
		    (string-match "^multipart/" (substring (car handle) 0 10))
		    (setq showed (or showed (hs-show-html-list handle))))))))
    showed))

(defun gnus-article-browse-html-article ()
  "View \"text/html\" parts of the current article with a WWW browser."
  (interactive)
  
  (save-window-excursion
    ;; Open raw article and select the buffer
    (gnus-summary-show-article t)
    (gnus-summary-select-article-buffer)
    (let ((parts (mm-dissect-buffer t t)))

      ;; If singlepart, enforce a list.
      (when (and (bufferp (car parts))
		 (stringp (car (mm-handle-type parts))))
	(setq parts (list parts)))

      ;; Process the list
      (unless (gnus-article-browse-html-parts parts)
	(gnus-error 3 "Mail doesn't contain a \"text/html\" part!"))

      (gnus-summary-show-article))))
    
(provide 'hs-show-html-mail)

;;; hs-show-html-mail.el ends here